/***********************************************************************
*
* Copyright (c) 2021-2022 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include <QtCore>
#include <QtGui>

class MainWindow : public QWidget
{
 public:
   MainWindow();

 private:
   void minDateChanged(const QDate &date);
   void maxDateChanged(const QDate &date);
   void selectedDateChanged();

   QCalendarWidget *calendar;

   QDateEdit *minCalendarDate;
   QDateEdit *maxCalendarDate;
   QDateEdit *currentDate;
};

MainWindow::MainWindow()
{
   setMinimumSize(700, 350);

   calendar = new QCalendarWidget;
   calendar->setMinimumDate(QDate(1950, 1, 1));
   calendar->setMaximumDate(QDate(2100, 1, 1));
   calendar->setGridVisible(true);

   QLabel *minLabel = new QLabel("Minimum Date:");
   minCalendarDate = new QDateEdit;
   minCalendarDate->setDisplayFormat("MMM d yyyy");
   minCalendarDate->setDate(calendar->minimumDate());
   minCalendarDate->setDateRange(calendar->minimumDate(), calendar->maximumDate());

   QLabel *maxLabel = new QLabel("Maximum Date:");
   maxCalendarDate = new QDateEdit;
   maxCalendarDate->setDisplayFormat("MMM d yyyy");
   maxCalendarDate->setDate(calendar->maximumDate());
   maxCalendarDate->setDateRange(calendar->minimumDate(), calendar->maximumDate());

   QLabel *currentLabel = new QLabel("Current Date:");
   currentDate = new QDateEdit();
   currentDate->setDisplayFormat("MMM d yyyy");
   currentDate->setDate(calendar->selectedDate());
   currentDate->setDateRange(calendar->minimumDate(), calendar->maximumDate());

   QPushButton *pb_1 = new QPushButton();
   pb_1->setText("Close");

   QGridLayout *grid1 = new QGridLayout();
   grid1->setContentsMargins(75, 45, 75, 25);
   grid1->addWidget(calendar, 0, 0, 1, 4);
   grid1->addItem(new QSpacerItem(0, 25), 1, 0, 1, 4);

   grid1->addWidget(minLabel, 2, 1);
   grid1->addWidget(minCalendarDate, 2, 2);

   grid1->addWidget(maxLabel, 3, 1);
   grid1->addWidget(maxCalendarDate, 3, 2);

   grid1->addWidget(currentLabel, 4, 1);
   grid1->addWidget(currentDate, 4, 2);

   QHBoxLayout *layout1 = new QHBoxLayout();
   layout1->addStretch();
   layout1->addWidget(pb_1);
   layout1->addStretch();

   QVBoxLayout *layoutMain = new QVBoxLayout(this);
   layoutMain->addLayout(grid1);
   layoutMain->addSpacing(25);
   layoutMain->addLayout(layout1);

   connect(minCalendarDate, &QDateEdit::dateChanged, this, &MainWindow::minDateChanged);
   connect(maxCalendarDate, &QDateEdit::dateChanged, this, &MainWindow::maxDateChanged);

   connect(calendar,    &QCalendarWidget::selectionChanged, this,     &MainWindow::selectedDateChanged);
   connect(currentDate, &QDateEdit::dateChanged,            calendar, &QCalendarWidget::setSelectedDate);

   connect(pb_1, &QPushButton::clicked, this, &QWidget::close);
}

void MainWindow::minDateChanged(const QDate &date)
{
   calendar->setMinimumDate(date);
   currentDate->setDateRange(calendar->minimumDate(), calendar->maximumDate());
}

void MainWindow::maxDateChanged(const QDate &date)
{
   calendar->setMaximumDate(date);
   currentDate->setDateRange(calendar->minimumDate(), calendar->maximumDate());
}

void MainWindow::selectedDateChanged()
{
   currentDate->setDate(calendar->selectedDate());
}

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   MainWindow *window = new MainWindow();
   window->show();

   return app.exec();
}

